/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/

/* *****************************************************************************
*
* functions.c
*
* all internal code for the APi functions
*
* by iri <iri@irizone.net>, november, 2010
*
* *****************************************************************************/



/* This is a classic include ... */
#include "main.h"


 
/* *****************************************************************************
*
* int sc_getFile (mmachine m);
*
* -> Scol function : example_getFile
* -> prototype Scol function : fun [S W] I
*
* Purpose : get a resource from the Web and write it to the local file
*
* MMpull (m) : pull (so, get and del) the last stage to the stack
* MMget (m, X) : get (and not del) the stage X at the stack
*
* MTOP : conversion from handle Scol to handle system
* PTOM : conversion from handle system to handle Scol
*
* MMpush (m, V) : add V to the stack
* MMset (m, X, V) : set V to the stage X
*
* This code pulls, respectively, the local file (stage 0, because last parameter)
* the url (stage 0 too, because the local file has been pulled).
* If one is NIL, the function returns 1 and stop
*
* Internal libcurl code
*
* If all right, returns 0, otherwise, returns 2 (libcurl init failed) or 3 (unable
* to open local file).
*
* *****************************************************************************/
int sc_getFile (mmachine m)
{
    int murl, mlocalfile, len;
    char * url, * localfile, * err = NULL;

    CURL * hCurl;     /* handle */
    CURLcode rCurl; /* result */
    FILE * fp = NULL;

    MMechostr (MSKDEBUG, "sc_getFile : entering ...\n");

    /* Get Scol parameters from the stack */
    mlocalfile = MTOP (MMpull (m));
    murl = MTOP (MMpull (m));

    /* test parameter */
    if ((mlocalfile == NIL) || (murl == NIL))
    {
        MMechostr (MSKDEBUG, "sc_getFile error : url or localfile is nil\n");
        MMpush (m, ITOM (1));
        return 0;
    }

    hCurl = curl_easy_init();
    if (hCurl)    /* init ok */
    {
        len = sizeof (char) * (MMsizestr (m, mlocalfile));
        localfile = (char *) malloc (len+1);
        strncpy (localfile, MMstartstr (m, mlocalfile), len);
        localfile[len] = '\0';

        fp = fopen (localfile, "w");
        if (fp == NULL)
        {
            curl_easy_cleanup (hCurl);
            free (localfile); localfile = NULL;
            MMpush (m, ITOM (3));
            return 0;
        }

        err = (char *) malloc (sizeof (char) * (CURL_ERROR_SIZE +1));
        len = sizeof (char) * (MMsizestr (m, murl));
        url = (char *) malloc (len+1);
        strncpy (url, MMstartstr (m, murl), len);
        url[len] = '\0';

        curl_easy_setopt (hCurl, CURLOPT_URL, url);
        curl_easy_setopt (hCurl, CURLOPT_WRITEFUNCTION, fwrite);
        curl_easy_setopt (hCurl, CURLOPT_WRITEDATA, (FILE *) fp);
        curl_easy_setopt (hCurl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
        curl_easy_setopt (hCurl, CURLOPT_ERRORBUFFER, err);
        rCurl = curl_easy_perform (hCurl);

        MMechostr (MSKDEBUG, "sc_getFile : err = %s ...\n", err);
        curl_easy_cleanup (hCurl);
        fclose (fp);

        free (localfile); localfile = NULL;
        free (url); url = NULL;
        free (err); err = NULL;

        /* returned Scol value */
        MMpush (m, ITOM (0));
        return 0;
    }

    MMpush (m, ITOM (2));
    return 0;
}




/* *****************************************************************************
*
* void sc_newUrlCBend (struct StrExample * strEx);
*
* internal function
*
* Called when the transmission is finished
*
* Called a Scol callback :
* - OBJbeginreflex :
* - - struct mmachine : the current Scol machine
* - - the Scol type object (see main.c)
* - - the system handle
* - - a specific flag (see main.c)
* - return 0 if ok. So, this return must be tested !
* 
* If ok, OBJcallreflex calls the Scol callback.
* Its arguments are :
* - - the current Scol machine
* - - the supplemental number Scol parameter
*
* A callback has always the linked Scol type object and an user parameter, at your
* convenience. If you want add it, you must enter the number here
*
* *****************************************************************************/
void sc_newUrlCBend (struct StrExample * strEx)
{
    mmachine m;
    CURL * hCurl;
    int cb;

    MMechostr (MSKDEBUG, "sc_newUrlCBend : entering ...\n");

    m = (mmachine) strEx->m2m;
    hCurl = (CURL *) strEx->hcurl;

    /* test and prepare the callback */
    cb = OBJbeginreflex (m, ObjCurl, (int) hCurl, CURL_CB_GET);
    if (cb)
    {
        MMechostr (0, "sc_newUrlCBend error : no callback defined ! errcode = %d\n", cb);
        return;
    }

    /* add fourth supplementals parameters : datas (NIl because the transmission is
    	ended), the size of theses (NIL too), the url and the hypothetic libcurl
    	error) */
    MMpush (m, NIL);
    MMpush (m, NIL);
    Mpushstrbloc (m,  strEx->url);
    Mpushstrbloc (m,  strEx->cerr);
    /* Call the Scol callback function */
    OBJcallreflex (m, 4);
    return;
}




/* *****************************************************************************
*
* size_t sc_newUrlCB (void * ptr, size_t size, size_t nmemb, void * p_strEx);
*
* internal function
*
* callback called by libcurl when a packet is received.
*
* Same thing than sc_newUrlCBend except the fourth additional parameter :
* - the datas (not NIL !)
* - the size of the datas
* - the url
* - the error message if any
*
* *****************************************************************************/
size_t sc_newUrlCB (void * ptr, size_t size, size_t nmemb, void * p_strEx)
{
    mmachine m;
    char * data = NULL;
    int cb;
    size_t n;
    struct StrExample * strEx;
    CURL * hCurl;

    MMechostr (MSKDEBUG, "sc_newUrlCB : entering ...\n");
    strEx = (struct StrExample *) p_strEx;

    m = (mmachine) strEx->m2m;
    hCurl = (CURL *) strEx->hcurl;

    cb = OBJbeginreflex (m, ObjCurl, (int) hCurl, CURL_CB_GET);
    if (cb)     /* no registred Scol callback with theses parameters */
    {
        MMechostr (0, "sc_newUrlCB error : no callback defined ! errcode = %d\n", cb);
        /* return -1 : libcurl stopps the connexion */
        return -1;
    }

    n = (sizeof (char) * size * nmemb);
    data = (char *) malloc (n+1);
    if (data == NULL)
        return -1;
    strncpy (data, ptr, n);
    data[n] = '\0';

    /* supplementals parameters */
    Mpushstrbloc (m,  data);            /* datas */
    MMpush (m, ITOM (size*nmemb));      /* its size */
    Mpushstrbloc (m,  strEx->url);      /* url (rappel) */
    Mpushstrbloc (m,  strEx->cerr);     /* hypothetic error message */
    /* launch the Scol callback */
    OBJcallreflex (m, 4);
    if (data)
        free (data);

    return size*nmemb;
}




/* *****************************************************************************
*
* int sc_newGetUrl (mmachine m);
*
* -> Scol function : example_newUrl
* -> Scol prototype : fun [Chn S S fun [ObjCurl u0 S I S S] u1 u0] ObjCurl
*
* Purpose : send a request and define a callback when any datas is received.
* A new Scol type object created.
*
* Scol Object creation :
*
* 1 - create this object (C code) i.e. its system handle
* 2 - allocate it
* 3 - store it to the Scol machine
* 4 - create it (scol handle)
*
* - allocation : MMalloc
* - - Scol machine
* - - the memory size
* - - the type of the object : TYPETAB or TYPEBUF (array or buffer)
*
* - storage : MMstore
* - - scol machine
* - - returned MMalloc's value
* - - specific Scol flag object
* - - system handle (int)
*
* - creation : OBJcreate
* - - scol machine
* - - Scol type Object
* - - handle system (int)
* - - parent Scol type object (may be NULL)
* - - parent handle system (may be NULL)
*
* Note : When you call OBJcreate, the channel MUST be at the stage 1 in the stack
* You can use INVERT macro to put it.
*
* Add a Scol callback function :
* Use OBJaddreflex :
* - the current Scol machine
* - the Scol typre object
* - a specific flag
*
* Note : user parameter should be at the stage 0 in the stack, the function name
* at the stage 1 and theScol object type at the stage 2
*
* *****************************************************************************/
int sc_newGetUrl (mmachine m)
{
    int mchannel, murl;
    int curltab, sizetab, tmp_res, n;
    CURLcode rCurl;
    struct StrExample strEx;

    MMechostr (MSKDEBUG, "sc_newGetUrl : entering ...\n");

    memset (strEx.cerr, '\0', CURL_ERROR_SIZE);

    mchannel = MMget (m, 4);
    if (mchannel == NIL)
    {
        MMechostr (0, "sc_newGetUrl error : channel is nil\n");
        MMpull (m);
        MMpush (m, NIL);
        return 0;
    }

    murl = MTOP (MMget (m, 3));
    if (murl == NIL)
    {
        MMechostr (0, "sc_newGetUrl error : url is nil\n");
        MMpull (m);
        MMpush (m, NIL);
        return 0;
    }

    /* Set the channel to the stage 0 */
    INVERT (m, 0, 4);

    /* libcurl global init */
    if (curl_global_init (CURL_GLOBAL_ALL) != CURLE_OK)
    { /* init falied */
        MMechostr (0, "sc_newGetUrl error : global init failed\n");
        MMpull (m);
        MMpush (m, NIL);
        return 0;
    }

    /* libcurl session init */
    strEx.hcurl = curl_easy_init();
    /* initialization OK, continue  */
    if (strEx.hcurl)
    {
        sizetab = sizeof (strEx.hcurl) + 1;
        /* Scol memory allocation */
        curltab = MMmalloc (m, sizetab, TYPETAB);
        if ((curltab == NIL)) /* memory fails */
        {
            MMechostr (0, "sc_newGetUrl error : insufficient memory\n");
            curl_easy_cleanup (strEx.hcurl);
            MMpull (m);
            MMpush (m, NIL);
            return 0;
        }
        /* store the system handle to the allocatede memory */
        MMstore (m, curltab, OBJCURL_HANDLE, (int) strEx.hcurl);
        /* push this new object to the stack (now, the channel is at the stage 1) 
        	the scol object is the returned value by the Scol function */
        MMpush (m, PTOM (curltab));
        /* Scol object creation */
        OBJcreate (m, ObjCurl, (int) strEx.hcurl, -1, -1);

        n = sizeof (char) * (MMsizestr (m, murl));
        strEx.url = (char *) malloc (n+1);
        strncpy (strEx.url, MMstartstr (m, murl), n);
        strEx.url[n] = '\0';

        strEx.m2m = m;

        INVERT (m, 0, 2);   /* system handle at the stage 2 */
        INVERT (m, 0, 4);   /* user pameter at the stage 0 */
        /* Add the callback function */
        OBJaddreflex (m, ObjCurl, CURL_CB_GET);

        curl_easy_setopt (strEx.hcurl, CURLOPT_URL, strEx.url);
        curl_easy_setopt (strEx.hcurl, CURLOPT_ERRORBUFFER, strEx.cerr);
        curl_easy_setopt (strEx.hcurl, CURLOPT_WRITEDATA, &strEx);
        curl_easy_setopt (strEx.hcurl, CURLOPT_WRITEFUNCTION, sc_newUrlCB);
        curl_easy_setopt (strEx.hcurl, CURLOPT_USERAGENT, "libcurl-agent/1.0");

        rCurl = curl_easy_perform (strEx.hcurl);
        sc_newUrlCBend (&strEx);

        curl_easy_cleanup (strEx.hcurl);
        curl_global_cleanup();

        free (strEx.url);

        return 0;
    }

    /* bad initialization */
    MMechostr (0, "sc_newGetUrl error : easy init failed\n");
    curl_global_cleanup();

    MMpull (m);
    MMpush (m, NIL);
    return 0;
}


/* *****************************************************************************
*
* int sc_delGetUrl (mmachine m);
*
* -> Scol function : example_delGetUrl
* -> Scol prototype : fun [ObjCurl] I
*
* Purpose : explicit destroy
*
* For this, call OBJdelTM with the handle system (int)
*
* *****************************************************************************/
int sc_delGetUrl (mmachine m)
{
    int obj;

    MMechostr (MSKDEBUG, "sc_delGetUrl entering ...\n");

    /* Get obect and test it */
    obj = MTOP (MMget (m, 0));
    if (obj == NIL)
    {
        MMechostr (MSKDEBUG, "sc_delGetUrl : object aleady destroyed\n");
        MMset (m, 0, NIL);
        return 0;
    }

    /* Use PTOM function because we need the handle system */
    OBJdelTM (m, OBJCURL_HANDLE, PTOM (obj));
    /* It's ok, return 0 */
    MMset (m, 0, 0);

    return 0;
}

