/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/




/* *****************************************************************************
*
* main.c
*
* basic Scol library
*
* by iri <iri@irizone.net>, november, 2010
*
* *****************************************************************************/



/* This is a classic include ... */
#include "main.h"





/* *****************************************************************************
*
* shared structures (kernel)
*
* See scol_plugin_xxx.h for theirs definitions
*
* Don't use theses variables into your code but you should always add them here
* and in your load function (see below)
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
cbmachine ww;
#endif
mmachine  mm;





/* *****************************************************************************
*
* int sc_increment (mmachine m);
*
* -> Scol function : example_increment
* -> Scol prototype : fun [I] I
*
* An internal Scol function take always one argument : the Scol machine structure 
* and return an integer
* This structure contains the Scol stack : usuals operations are possibles :
* MMpull -> pull the last stage (stage 0). So, get it and delete it
* MMget -> get the contnt of a stage : MMget (m, 2) get to the stage 2
* MMpush -> push to the stack, in the last stage (stage 0). Previous stage 0 becomes 1
* MMset -> set (replace) the content of any stage
*
* Others functions exist, we shall see later.
*
* Usuals macros :
* MTOI : convert any Scol integer to a system integer
* ITOM : convert any system integer to a Scol integer
* MTOP : convert any Scol handle to a system handle
* PTOM : convert any system handle to a Scol handle
*
*
* This function increments a value. So, it takes an ineger and return the same 
* integer + 1
*
* First : get or pull this integer to the Scol stack
* Second : increment it
* Third : return the new value : push or set it to the Scol stack
*
* *****************************************************************************/
int sc_increment (mmachine m)
{
    int mnumber;

    /* Debug / log function : write a message to the console and/or the log file.
    Masks are availables :
    (nothing)   -> 0
    MSKFOO      -> 1
    MSKRUNTIME  -> 2
    MSKWARNING  -> 4
    MSKTRACE    -> 8
    MSKDEBUG    -> 16 */
    MMechostr (MSKDEBUG, "sc_increment: entering\n");

    /* Pull the last stage of the Scol stack.
    Note : we could make it with MMget : mnumber = MTOI (MMget (m, 0));. Here, it's
    the same thing */
    mnumber = MTOI (MMpull (m));

    mnumber++;

    /* Push the new value to the Scol stack = returned value by the Scol function
    If we used MMget, we make it : MMset (m, 0, ITOM (mnumber)); */
    MMpush (m, ITOM (mnumber));

    return 0;
}





/* *****************************************************************************
*
* API definition
*
* 1- We define all new Scol element. These elements may be called from any Scol 
* script.
* 2- All internal functions or NULL
* 3- The type or the number of the arguments (if functions) 
* 4- The Scol prototypes
*
* The size of each array / function must be the same (here 'EXAMPLE_PKG_NB').
*
* *****************************************************************************/
char* example_name[EXAMPLE_PKG_NB] =
{
    "example_increment"
};

int (*example_fun[EXAMPLE_PKG_NB]) (mmachine m) =
{
    sc_increment
};

int example_narg[EXAMPLE_PKG_NB] =
{
    1
};

char* example_type[EXAMPLE_PKG_NB] =
{
    "fun [I] I"
};





/* *****************************************************************************
*
* int SCOLinitEXAMPLEclass (mmachine m);
*
* Load the new Scol API
* It takes one argument : the current Scol machine structure
*
* 'PKhardpak' is an Scol internal function to load any new API (it exists an other
* function in the same vm/src/kernel/hardload.c file, less used). It takes 7 arguments :
* 1st- the Scol machine structure
* 2nd- an internal name for this package
* 3rd- the number of elements in the new API
* 4th, 5th, 6th and 7th : the API definition (see above)
* Return 0 if OK
* 
* *****************************************************************************/
int SCOLinitEXAMPLEclass (mmachine m)
{
    int k = 0;

    MMechostr (MSKDEBUG, "SCOLinitEXAMPLEclass library : loading\n");

    k = PKhardpak (m, "EXAMPLEengine", EXAMPLE_PKG_NB, example_name, example_fun, example_narg, example_type);
    return k;
}







/* *****************************************************************************
*
* Load the library by the kernel
*
* The name of this function must be defined in the usm.ini file :
* 
* plugin plugins/library.dll load_function_name free_function_name (under MS Windows)
* plugin ./plugins/library.so load_function_name free_function_name (under GNU / Linux)
*
* example : plugin plugins/libexample.dll SCOLloadEXAMPLE SCOLfreeEXAMPLE
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/

#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLloadEXAMPLE (mmachine m, cbmachine w)
{
    int k = 0;

    /* Must set it ! Shared structures (kernel) */
    ww = w;
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#elif defined linux || defined __linux
int SCOLloadEXAMPLE (mmachine m)
{
    int k = 0;

    /* Must set it ! Shared structure (kernel) */
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#else
#error no platform supported
#endif





/* *****************************************************************************
*
* Free the library (optional)
*
* If any, its name must be in the usm.ini file (see above)
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLfreeEXAMPLE ()
#elif defined linux || defined __linux
int SCOLfreeEXAMPLE ()
#else
#error no platform supported
#endif
{
    MMechostr(MSKDEBUG, "\nEXAMPLE library release !\n");
    return 0;
}
