/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/

/* *****************************************************************************
*
* main.c
*
* API definition
*
* by iri <iri@irizone.net>, november, 2010
*
* *****************************************************************************/


/* standard include */
#include "main.h"



/* *****************************************************************************
*
* shared structure (kernel)
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
cbmachine ww;
#endif
mmachine  mm;




/* *****************************************************************************
*
* int ObjCurlDestroy (mmachine m, int handsys, int mobj);
*
* internal Scol function : implicit callback destroy
*
* *****************************************************************************/
int ObjCurlDestroy (mmachine m, int handsys, int mobj)
{
    CURL * c;

    MMechostr(MSKDEBUG, "ObjCurlDestroy: entering ...\n");

	/* return the object handle */
    c = (CURL *) MMfetch (m, MTOP (mobj), OBJCURL_HANDLE);
    if (c == NULL)
    {
        MMechostr(MSKDEBUG, "ObjCurlDestroy : object already destroyed\n");
        return 0;
    }
    /*curl_easy_cleanup (c);*/

	/* unregister this object to the Scol machine */
    MMstore (m, MTOP (mobj), OBJCURL_HANDLE, (int) NULL);
    MMechostr(MSKDEBUG, "ObjCurlDestroy: object has been destroyed\n");

    return 0;
}




/* *****************************************************************************
*
* API definition
*
* - 1 : API names : new types, functions, flags ...
* - 2 : internals functions / values
* - 3 : type or the number of the arguments (functions only) 
* - 4 : Scol prototypes
*
* *****************************************************************************/
char* example_name[EXAMPLE_PKG_NB] =
{
    /* new Scol type */
    "ObjCurl",

    /* new Scol functions */
    "example_getFile",
    "example_newUrl",
    "example_delGetUrl"
};

/* Pointers to C functions that manipulate the VM for each scol function previously 
	defined */
int (*example_fun[EXAMPLE_PKG_NB]) (mmachine m) =
{
    NULL,

    sc_getFile,
    sc_newGetUrl,
    sc_delGetUrl
};

/* Nb of arguments of each scol function or type (TYPTYPE (new Scol type), TYPVAR
	(numeric constante)). Others are rarely used, or unused */
int example_narg[EXAMPLE_PKG_NB] =
{
    TYPTYPE,

    2,
    5,
    1
};

/* Scol prototypes */
char* example_type[EXAMPLE_PKG_NB] =
{
    NULL,

    "fun [S W] I"
    "fun [Chn S S fun [ObjCurl u0 S I S S] u1 u0] ObjCurl",
    "fun [ObjCurl] I"
};



/* *****************************************************************************
*
* int SCOLinitEXAMPLEclass (mmachine m);
*
* Load the new Scol api. Define it
*
* Return 0 if ok otherwise the error code
*
* ******************************************************************************/
int SCOLinitEXAMPLEclass (mmachine m)
{
    int k = 0;

    MMechostr (MSKDEBUG, "SCOLinitEXAMPLEclass library : loading\n");

    /* Register a new Scol type and define the destroy callback */
    ObjCurl  = OBJregister (CURL_RFL_NB, 1, ObjCurlDestroy, "ObjCurlType");
    /* add a new event .. to do ...*/
    OBJgetUserEvent();

    k = PKhardpak (m, "EXAMPLEengine", EXAMPLE_PKG_NB, example_name, example_fun, example_narg, example_type);
    return k;
}





/* *****************************************************************************
*
* Load the library by the kernel
*
* The name of this function must be  in the usm.ini file :
* 
* plugin plugins/library.dll load_function_name free_function_name (under MS Windows)
* plugin ./plugins/library.so load_function_name free_function_name (under GNU / Linux)
*
* example : plugin plugins/libexample.dll SCOLloadEXAMPLE SCOLfreeEXAMPLE
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLloadEXAMPLE (mmachine m, cbmachine w)
{
    int k = 0;
    /* Must set it ! Shared structures (kernel) */
    ww = w;
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#elif defined linux || defined __linux
int SCOLloadEXAMPLE (mmachine m)
{
    int k = 0;
    /* Must set it ! Shared structure (kernel) */
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#else
#error no platform supported
#endif




/* *****************************************************************************
*
* Free the library (optional)
*
* If any, its name must be in the usm.ini file (see above)
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/
#if defined linux || defined __linux
int SCOLfreeEXAMPLE ()

#elif defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLfreeEXAMPLE ()

#else
#error no platform supported
#endif

{
    MMechostr(MSKDEBUG, "\nEXAMPLE library release !\n");
    return 0;
}
