/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/




/* *****************************************************************************
*
* main.c
*
* basic Scol library
*
* by iri <iri@irizone.net>, november, 2010
*
* *****************************************************************************/



/* This is a classical include ... */
#include "main.h"





/* *****************************************************************************
*
* shared structures (kernel)
*
* See scol_plugin_xxx.h for theirs definitions
*
* Don't use theses variables into your code but you should always add them here
* and in your load function (see below)
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
cbmachine ww;
#endif
mmachine  mm;







/* *****************************************************************************
*
* int sc_increment (mmachine m);
*
* -> Scol function : example_increment
* -> Scol prototype : fun [I] I
*
* See example 1
*
* *****************************************************************************/
int sc_increment (mmachine m)
{
    int mnumber;

    MMechostr (MSKDEBUG, "sc_increment : entering\n");

    mnumber = MTOI (MMget (m, 0));
    mnumber++;
    MMset (m, 0, ITOM (mnumber));
    return 0;
}






/* *****************************************************************************
*
* int sc_hello (mmachine m);
*
* -> Scol function : example_hello
* -> Scol prototype : fun [S] S
*
* As a famous "Hello World" ! In fact, it is a welcome message.
* A name in entry, a "Hello name" in exit.
*
*  To pull / get a string, follow these instructions :
* 1- MMpull or MMget to retreive the pointer (with MTOP)
* 2- Set the cursor to the start : MMstartstr
* 3- If needed, MMsiezstr returns the size ofthe string (memory allocation by example)
*
* To push a string to the Scol stack, use Mpushstrbloc !
*
* *****************************************************************************/
int sc_hello (mmachine m)
{
    int menter, n;
    char * enter = NULL;
    char * result = NULL;

    MMechostr (MSKDEBUG, "sc_hello : entering\n");

    menter = MTOP (MMpull (m))
    enter = MMstartstr (m, menter);

    n = sizeof (char) * (MMsizestr (m, menter));
    result = (char *) malloc (sizeof (char) * (n+7)); /* memory allocation */
    sprintf (result, "%s %s", "Hello", enter);

    Mpushstrbloc (m, result);
    free (result);
    return 0;
}







/* *****************************************************************************
*
* int sc_division (mmachine m);
*
* -> Scol function : example_calcul
* -> Scol prototype : fun [[I I]] [I I]
*
* Purpose : manage a tuple
*
* To get a tuple from the Scol stack :
*
* MMpull or MMget -> the tuple
* MMfetch X -> value X in the tuple
*
* example :
* tuple = MMpull (m);
* value_1 = MMfetch (m, tuple, 0); -> first element in the tuple
* value_2 = MMfetch (m, tuple, 1); -> second element in the tuple
* value_3 = MMfetch (m, tuple, 2); -> third element in the tuple
* ...
*
* To push a tuple to the Scol stack :
*
* push each element (MMpush)
* push the number of elements (MMpush)
* create the tuple (MBdeftab)
*
* example :
* MMpush (m, value_1);
* MMpush (m, value_2);
* MMpush (m, value_3);
* MMpush (m, ITOM (3));
* MBdeftab (m);
*
* *****************************************************************************/
int sc_division (mmachine m)
{
    int mtuple, mdividende, mdiviseur;
    int quotient, reste;

    MMechostr (MSKDEBUG, "sc_division : entering\n");

    mtuple = MTOI (MMpull (m));                     /* 1 */
    mdividende = MTOI (MMfetch (m, mtuple, 0));     /* 2 */
    mdiviseur = MTOI (MMfetch (m, mtuple, 1));      /* 2 */

    if (mdiviseur == 0)     /* division by zero */
        return MMpush (m, NIL);

    quotient = mdividende / mdiviseur;
    reste = mdividende % mdiviseur;

    MMpush (m, ITOM (quotient));                    /* 3 */
    MMpush (m, ITOM (reste));                       /* 3 */
    MMpush (m, ITOM (2));                           /* 4 */
    return MBdeftab (m);                            /* 5 */
}











/* *****************************************************************************
*
* int sc_calcul_list (mmachine m);
*
* -> Scol function : example_list
* -> Scol prototype : fun [[I r1] I I] [I r1]
*
* Purpose : manage lists
*
* With this function :
* in : 7 :: 5 :: 2 :: nil, 10, EXAMPLE_MORE
* out : 17 :: 15 :: 12 :: nil
* in : 7 :: 5 :: 2 :: nil, 10, EXAMPLE_LESS
* out : -3 :: -5 :: -8 :: nil
*
* To pull (or get) a list :
* In Scol, a list is a tuple :
* list = a :: b ::c :: d :: e :: nil;
* is equal to :
* list = [a [b [c [d [e nil]]]]];
* so, list = [first next];
* next = [second next2]
* next2 = [third next3] ....
*
* - pull the list : MMpush
* - make a loop (while)
* - - while the list is not nil
* - - - fetch the element : MMfetch
* - - - if this element is a tuple or a list, same thing, new loop ...
*
* To push a list to the Scol stack :
*
* - A loop : WHILE to push each element
* - Push the last  element = NIL ! (always !)
* - Create all tuple (remerber, above)
* - it's ok.
*
* *****************************************************************************/
int sc_calcul_list (mmachine m)
{
    int mlist_enter, mnumber, mflag;
    int element_list, i, compteur = 0;

    MMechostr (MSKDEBUG, "sc_calcul_list : entering\n");

    mflag = MTOI (MMpull (m));
    mnumber = MTOI (MMpull (m));
    mlist_enter = MMpull (m)>>1;

    while (mlist_enter != NIL)
    {
         element_list = MTOI (MMfetch (m, mlist_enter, 0));
         if (element_list != NIL)
         {
             MMpush (m, ITOM (element_list + (mnumber * mflag)));
             compteur++;
         }
         mlist_enter = MTOI (MMfetch(m, mlist_enter, 1));
    }

    MMpush (m, NIL);

    for (i = 0; i < compteur; i++)
    {
        MMpush (m, ITOM (2));
        MBdeftab (m);
    }
    return 0;
}







/* *****************************************************************************
*
* API definition
*
* 1- We define all new Scol element. These elements may be called from any Scol 
* script.
* 2- All internal functions or NULL
* 3- The type or the number of the arguments (if functions) 
* 4- The Scol prototypes
*
* The size of each array / function must be the same (here 'EXAMPLE_PKG_NB').
*
* *****************************************************************************/
char* example_name[EXAMPLE_PKG_NB] =
{
    /* functions */
    "example_increment",
    "example_hello",
    "example_calcul",
    "example_list",

    /* flags */
    "EXAMPLE_LESS",
    "EXAMPLE_MORE"
};


int (*example_fun[EXAMPLE_PKG_NB]) (mmachine m) =
{
    sc_increment,
    sc_hello,
    sc_division,
    sc_calcul_list,

    (void*) ((-1)*2),
    (void*) (1*2)
};


int example_narg[EXAMPLE_PKG_NB] =
{
    1,
    1,
    1,
    3,

    TYPVAR,
    TYPVAR
};


char* example_type[EXAMPLE_PKG_NB] =
{
    "fun [I] I",
    "fun [S] S",
    "fun [[I I]] [I I]",
    "fun [[I r1] I I] [I r1]",

    "I",
    "I"
};








/* *****************************************************************************
*
* int SCOLinitEXAMPLEclass (mmachine m);
*
* Load the new Scol API
* It takes one argument : the current Scol machine structure
*
* 'PKhardpak' is an Scol internal function to load any new API (it exists an other
* function in the same vm/src/kernel/hardload.c file, less used). It takes 7 arguments :
* 1st- the Scol machine structure
* 2nd- an internal name for this package
* 3rd- the number of elements in the new API
* 4th, 5th, 6th and 7th : the API definition (see above)
* Return 0 if OK
* 
* *****************************************************************************/
int SCOLinitEXAMPLEclass (mmachine m)
{
    int k = 0;

    MMechostr (MSKDEBUG, "SCOLinitEXAMPLEclass library : loading\n");

    k = PKhardpak (m, "EXAMPLEengine", EXAMPLE_PKG_NB, example_name, example_fun, example_narg, example_type);
    return k;
}







/* *****************************************************************************
*
* Load the library by the kernel
*
* The name of this function must be defined in the usm.ini file :
* 
* plugin plugins/library.dll load_function_name free_function_name (under MS Windows)
* plugin ./plugins/library.so load_function_name free_function_name (under GNU / Linux)
*
* example : plugin plugins/libexample.dll SCOLloadEXAMPLE SCOLfreeEXAMPLE
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/
#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLloadEXAMPLE (mmachine m, cbmachine w)
{
    int k = 0;
    /* Must set it ! Shared structures (kernel) */
    ww = w;
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#elif defined linux || defined __linux
int SCOLloadEXAMPLE (mmachine m)
{
    int k = 0;
    /* Must set it ! Shared structure (kernel) */
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#else
#error no platform supported
#endif





/* *****************************************************************************
*
* Free the library (optional)
*
* If any, its name must be in the usm.ini file (see above)
*
* Note : this function is not the same under MS Windows and GNU / Linux
*
* *****************************************************************************/
#if defined linux || defined __linux
int SCOLfreeEXAMPLE ()

#elif defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLfreeEXAMPLE ()

#else
#error no platform supported
#endif

{
    MMechostr(MSKDEBUG, "\nEXAMPLE library release !\n");
    return 0;
}
